import { Type } from "./type";
/**
 * Describes the mode used to abbreviate large numbers
 */
export declare enum DataAbbreviationMode {
    /**
     * Specifies abbreviation that is automatically resolves to shared abbreviation between multiple numbers, 2.5K for 2,500 and 2500K for 2,500,000
     */
    Auto = 0,
    /**
     * Specifies independent abbreviation for multiple numbers, 2.5K for 2,500 and 2.5M for 2,500,000
     */
    Independent = 1,
    /**
     * Specifies shared abbreviation between multiple numbers, 2.5K for 2,500 and 2500K for 2,500,000
     */
    Shared = 2,
    /**
     * Specifies number abbreviation to thousands, 2.5K for 2,500
     */
    Kilo = 3,
    /**
     * Specifies number abbreviation to millions, 2.5M for 2,500,000
     */
    Million = 4,
    /**
     * Specifies number abbreviation to billion, 2.5B for 2,500,000,000
     */
    Billion = 5,
    /**
     * Specifies number abbreviation to trillion, 2.5T for 2,500,000,000,000
     */
    Trillion = 6,
    /**
     * Specifies number abbreviation to quadrillion, 2.5Q for 2,500,000,000,000,000
     */
    Quadrillion = 7,
    /**
     * Specifies default abbreviation
     */
    Unset = 8,
    /**
     * Specifies no abbreviation of numbers
     */
    None = 9
}
/**
 * @hidden
 */
export declare let DataAbbreviationMode_$type: Type;
